/*-
 * #%L
 * Smile CDR - CDR
 * %%
 * Copyright (C) 2016 - 2024 Smile CDR, Inc.
 * %%
 * All rights reserved.
 * #L%
 */
package com.example.ips;

import ca.uhn.fhir.jpa.ips.api.ISectionResourceSupplier;
import ca.uhn.fhir.jpa.ips.api.Section;
import ca.uhn.fhir.jpa.ips.jpa.DefaultJpaIpsGenerationStrategy;
import ca.uhn.fhir.jpa.ips.jpa.JpaSectionSearchStrategyCollection;
import org.hl7.fhir.r4.model.Communication;
import org.hl7.fhir.r4.model.Coverage;

import java.util.List;

/**
 * This class defines the generation strategy for your custom IPS
 * document generator.
 */
public class CustomIpsGenerationStrategy extends DefaultJpaIpsGenerationStrategy {

	/**
	 * If you want to support multiple generation strategies in a single server,
	 * then each one should return a different profile URL. The user can request
	 * which specific generator to use by including the "profile" parameter for the
	 * $summary operation. If you only want to include a single strategy, this
	 * method is not needed (the implementation here is just a copy of the
	 * implementation in the superclass)
	 */
	@Override
	public String getBundleProfile() {
		return "http://hl7.org/fhir/uv/ips/StructureDefinition/Bundle-uv-ips";
	}

	/**
	 * We override the default addSections method in order to specify our
	 * own sections. Note that the order matters here, so whatever
	 * order we add sections will ultimately be the order that those
	 * sections appear in the IPS.
	 */
	@Override
	protected void addSections() {

		// For our custom document, we'll add some of the default
		// built-in sections, but not all of them
		super.addJpaSectionAllergyIntolerance();
		super.addJpaSectionMedicationSummary();
		super.addJpaSectionProblemList();
		super.addJpaSectionImmunizations();
		super.addJpaSectionProcedures();

		// And we'll add two custom sections of our own
		this.addJpaSectionInsurance();
		this.addSectionPatientNotes();
	}

	/**
	 * Add a section definition for an Insurance section. This section
	 * includes active Coverage resources for the patient. It is a JPA
	 * section, meaning that it fetches its resources from the
	 * FHIR repository (i.e. from a FHIR Storage module).
	 */
	private void addJpaSectionInsurance() {
		Section.SectionBuilder sectionBuilder = Section.newBuilder();

		// This is the section title, for Composition.section.title
		sectionBuilder.withTitle("Insurance");

		// This is the code that will identify the section in
		// Composition.section.code
		sectionBuilder
				.withSectionSystem("http://example.com/fhir/IPS/Section")
				.withSectionCode("ins")
				.withSectionDisplay("Insurance");

		// This is used to determine the narrative generator template to use,
		// and we'll refer to it in the GenerationStratregy too
		sectionBuilder.withProfile("https://example.com/fhir/IPS/Profile#Insurance");

		// Add any resource types that should appear in this section
		sectionBuilder.withResourceType(Coverage.class);

		Section section = sectionBuilder.build();

		JpaSectionSearchStrategyCollection searchStrategyCollection = JpaSectionSearchStrategyCollection.newBuilder()
				.addStrategy(Coverage.class, new InsuranceJpaSectionSearchStrategy())
				.build();

		addJpaSection(section, searchStrategyCollection);
	}

	/**
	 * Add a section definition for a Patient Notes section. This section is a
	 * non-JPA section, meaning that it does not fetch its data from a repository,
	 * and instead relies on custom code specified in the class
	 * PatientNotesSectionResourceSupplier.
	 */
	private void addSectionPatientNotes() {
		Section.SectionBuilder sectionBuilder = Section.newBuilder();

		// This is the section title, for Composition.section.title
		sectionBuilder.withTitle("Patient Notes");

		// This is the code that will identify the section in
		// Composition.section.code
		sectionBuilder
				.withSectionSystem("http://example.com/fhir/IPS/Section")
				.withSectionCode("notes")
				.withSectionDisplay("Patient Notes");

		// This is used to determine the narrative generator template to use,
		// and we'll refer to it in the GenerationStratregy too
		sectionBuilder.withProfile("https://example.com/fhir/IPS/Profile#PatientNotes");

		// Add any resource types that should appear in this section
		sectionBuilder.withResourceType(Communication.class);

		Section section = sectionBuilder.build();
		ISectionResourceSupplier resourceSupplier = new PatientNotesSectionResourceSupplier();

		addSection(section, resourceSupplier);
	}

	/**
	 * This method returns the collection of property files for IPS section narrative
	 * generation. Here we return the default set, but with a custom addition.
	 */
	@Override
	public List<String> getNarrativePropertyFiles() {
		List<String> retVal = super.getNarrativePropertyFiles();
		retVal.add("classpath:com/example/ips/ips-narratives.properties");
		return retVal;
	}
}
